<?php
// $Id: project_forecast.admin.inc,v 1.2 2008/07/17 18:42:14 jpetso Exp $
/**
 * @file
 * Project Forecast - Estimate completion dates for your project's tasks.
 *
 * This file contains the general parts of the administrative user interface.
 * The "Task filters" page has been outsourced to the .admin.filters.inc file
 * because of its complexity.
 *
 * Copyright 2007 by Jakob Petsovits ("jpetso", http://drupal.org/user/56020)
 */

/**
 * Form callback for "admin/project/forecast[/relationships]":
 * Let the admin select views of issues (plus arguments)
 * and other node relationships.
 */
function project_forecast_admin(&$form_state) {
  $views = array();
  $all_views = views_get_all_views();
  foreach ($all_views as $view) {
    $views[$view->name] = $view->name;
  }
  $original_tasks_view = variable_get('project_forecast_tasks_view', '');
  $form_state['original_tasks_view'] = $original_tasks_view;

  // Notify the user about missing additional filters, as it's easy to miss.
  if (!empty($original_tasks_view)) {
    $filters = variable_get('project_forecast_task_filters_open', array());
    if (empty($filters)) {
      drupal_set_message(t("You have not configured any additional filters for reducing this view to only open tasks. Please visit the !task_filters page for adding one or more of those.", array('!task_filters' => l(t('Task filters'), 'admin/project/forecast/task-filters'))), 'warning');
    }
  }

  $hours_providers = value_provider_options('hours');
  $nid_providers = value_provider_options('nid');
  $uid_providers = value_provider_options('uid');

  $form = array();

  $form['general'] = array(
    '#type' => 'fieldset',
    '#title' => t('Task retrieval'),
    '#description' => t('Define how the list of tasks to be analyzed (...or issues, cases, bugs) is going be retrieved.'),
    '#collapsible' => TRUE,
    '#collapsed' => (variable_get('project_forecast_tasks_view', '') != ''),
  );
  $form['general']['project_forecast_tasks_view'] = array(
    '#type' => 'select',
    '#title' => t('View of all tasks'),
    '#options' => array_merge(array('' => t('please select a view')), $views),
    '#required' => TRUE,
    '#default_value' => $original_tasks_view,
    '#description' => t('Choose the "Views module" view that selects all task nodes. The forecast will automatically filter this view according to its needs, using the other information that you provide here. For open tasks, the view\'s "sort criteria" section will be used to determine in which order the users are expected to complete these tasks.'),
  );
  $form['general']['project_forecast_tasks_view_args'] = array(
    '#type' => 'textfield',
    '#title' => t('View arguments'),
    '#default_value' => variable_get('project_forecast_tasks_view_args', ''),
    '#required' => FALSE,
    '#description' => t('Provide a comma separated list of arguments to pass to the above view.'),
  );
  $form['properties'] = array(
    '#type' => 'fieldset',
    '#title' => t('Task properties'),
    '#description' => t('Define which data from the task nodes is going to be used by the forecast.'),
    '#collapsible' => TRUE,
    '#collapsed' => FALSE,
  );
  $form['properties']['project_forecast_timeneed_provider'] = array(
    '#type' => 'select',
    '#title' => t('Field for the task\'s time need'),
    '#options' => $hours_providers,
    '#required' => TRUE,
    '#default_value' => variable_get('project_forecast_timeneed_provider', ''),
    '#description' => t('The project forecast needs to know how many time is still required for a given task, and this information is extracted from the field that you need to specify here. Tasks without this field are assumed to have a zero-length time need.'),
  );
  $form['properties']['project_forecast_project_nid_provider'] = array(
    '#type' => 'select',
    '#title' => t('Field for the project node that contains the task'),
    '#options' => $nid_providers,
    '#required' => TRUE,
    '#default_value' => variable_get('project_forecast_project_nid_provider', ''),
    '#description' => t('The project forecast needs to know which project contains the given task, and this information is extracted from the field that you need to specify here.'),
  );
  $form['properties']['project_forecast_assigned_uid_provider'] = array(
    '#type' => 'select',
    '#title' => t('Field for the user who is assigned to this task'),
    '#options' => $uid_providers,
    '#required' => TRUE,
    '#default_value' => variable_get('project_forecast_assigned_uid_provider', ''),
    '#description' => t('The project forecast needs to know which user a task is assigned to, in order to process the tasks separately for each user. This field is required to provide this information.'),
  );

  $form = system_settings_form($form);
  $form['#submit'][] = 'project_forecast_admin_submit';

  return $form;
}

function project_forecast_admin_submit($form, &$form_state) {
  if ($form_state['values']['project_forecast_tasks_view'] != $form_state['original_tasks_view']) {
    // New view, new filters - the old ones probably don't fit.
    variable_del('project_forecast_task_filters_open');
  }
  project_forecast_recalculate_all();
}
